/* -*-C-*- strtod.c */
/*-->strtod*/
/**********************************************************************/
/****************************** strtod ********************************/
/**********************************************************************/

/*
************************************************************************

====================
IMPLEMENTATION NOTES
====================

This is a careful implementation of Standard C strtod(), and provided
that pow() is accurate (see comments in the final scaling section
below), will be accurate over the entire floating-point range,
including near the underflow and overflow limits.

The worst-case error in the result is (2*p + 2.5) ulps, where p
is the error (in ulps) in pow().

!!!! REVISE CODE AND ERROR ESTIMATES !!!!

There are certain unavoidable architectural assumptions in this
implementation (see the #define statements below), but they are broad
enough to cover every reasonable programmable computer from PCs to
mainframes to supercomputers.


=======================
ACCURACY CONSIDERATIONS
=======================

The inner of loop of strtod() looks like

	fp_value <- 10*fp_value + digit_value

This computation is EXACT until the number of bits in the RHS result
exceeds the number that can be stored in the fractional part of a
floating-point number.

At that point, rounding errors from each of the multiply and add
operations begin to accumulate.  Since the digit_value contribution to
the sum is then essentially zero, the major source of error arises
from the multiplication.  For each additional digit processed, the
worst-case error increases by an amount that varies from 0.5ulp (with
rounding arithmetic) to 1 ulp (with truncating arithmetic), although
in practice, the error accumulation is not as severe as this.

Care must be taken to avoid premature overflow of the accumulating
fp_value; see the COMMON_BASE scaling code below.

Once the fp_value is accumulated, it must be multiplied by an
appropriate power of ten from any trailing E+nn exponent field, and
once again, scaling of the exponent may be necessary to avoid
premature underflow or overflow.  Rather than compute the power of ten
directly, we call pow() to do so, in the expectation of minimizing the
error.

Note that an exact conversion according to the rules of IEEE
arithmetic would use `round to nearest', with ties broken by `round to
even'.  To understand the implication of this requirement, consider a
binary representation of the result, with . marking the significance
cutoff:

	...xxxxx0.0xxxxxx....x	-> ...xxxxx0 (round down)
	...xxxxx0.1000000....0	-> ...xxxxx0 (round to even (down))
	...xxxxx0.1000000...01	-> ...xxxxx0 (round up)
	...xxxxx1.0xxxxxx....x	-> ...xxxxx1 (round down)
	...xxxxx1.1000000....0	-> ...yyyyy0 (round to even (up))
	...xxxxx1.1000000....1	-> ...yyyyy0 (round up)

The first round-to-even case (bit just before cutoff is 0) requires
exact computation to the last bit, showing that correctly-rounded
results cannot be obtained in all cases unless the computation is done
exactly, which requires multiple precision arithmetic.  The cases
where the bit immediately before the cutoff is 1 can be correctly
rounded by examining only the immediately following bit.


============================
FLOATING-POINT NUMBER FORMAT
============================

Any leading whitespace is ignored, and the number recognized is
defined by the grammar (Standard C, Section A.1.1.4)

floating-constant:
	sign-opt fractional-constant exponent-part
	sign-opt digit-sequence exponent-part

digit-sequence:
	digit
	digit digit-sequence

exponent-part:
	e sign-opt digit-sequence
	E sign-opt digit-sequence

fractional-constant:
	. digit-sequence
	digit-sequence . digit-sequence
	digit-sequence .

sign-opt:
	<nil>
	+
	-

digit:	one of
	0 1 2 3 4 5 6 7 8 9


================
EXACT CONVERSION
================

Interestingly, exact conversion is possible, though complicated;
for details, see

@Article{Clinger:floating-point-input,
  author = 	"William D. Clinger",
  title = 	"How to Read Floating Point Numbers Accurately",
  journal = 	SIGPLAN,
  year = 	"1990",
  volume = 	"25",
  number = 	"6",
  pages = 	"92--101",
  month = 	jun,
}

@Article{Steele:floating-point-output,
  author = 	"Guy L. {Steele Jr.} and Jon L. White",
  title = 	"How to Print Floating-Point Numbers Accurately",
  journal = 	SIGPLAN,
  year = 	"1990",
  volume = 	"25",
  number = 	"6",
  pages = 	"112--126",
  month = 	jun,
}

plus these corrections:

>> ...
>> From: validgh!sun!Think.COM!gls@Sun.COM (Guy Steele)
>> Date: Wed, 27 Jun 90 11:55:36 EDT
>> In-Reply-To: Marianne Mueller's message of Tue, 26 Jun 90 17:17:15 PDT
>>              <9006270017.AA00474@puffin.Eng.Sun.COM>
>> Subject: 3 corrections to Steele's SIGPLAN 90 paper
>>
>>    Date: Tue, 26 Jun 90 17:17:15 PDT
>>    From: mrm@Eng.Sun.COM (Marianne Mueller)
>>
>> Thank you for a quite accurate summary.  It is heartening to
>> know that someone actually listened and cared!  A few minor
>> emendations appear below.
>>
>>    At the SIGPLAN 90 conference last week, Guy Steele pointed out that an
>>    intrepid pre-conference implementation of what's stated in the paper
>>    revealed 3 mistakes.
>>
>>    1.  Table 9 (page 125)
>>
>>            for             -1:USER!("");
>>            substitute      -1:USER!("0"); and delete the comment
>>
>>
>>    2.  Table 10 (page 125)
>>
>>            for          fill(-k, "0")
>>            substitute   fill(-k-1, "0")
>>
>>    3.  Table 5 (page 124)
>>            insert               k <-- 0  after assertion
>>
>> And also delete k <-- 0 from Table 6.
>>
>>    The paper is "How to Print Floating-Point Numbers Accurately", Guy L.
>>    Steele (gls@think.com) and Jon L. White (jonl@lucid.com), pp 112-126,
>>    Proceedings of SIGPLAN 90, SIGPLAN Notices Volume 26, Number 6, June
>>    1990.
>>
>>    The presentation was interesting since he didn't try to go through all
>>    the algorithms blow by blow but instead focused on the motivation and
>>    what it is he's trying to accomplish.  So audience participation was
>>    high!
>>
>>    Here's a transcription of some notes I took.  If I got part of this
>>    wrong maybe gls@think can correct me.
>>
>>    The base conversion problems are:
>>            1. Avoid scaling
>>
>>                    See Matula[68] for a discussion of why we want to
>>                    avoid scaling; the idea is that if L and L' are
>>                    consecutive floating point numbers, then
>>                    L * (10**k) = L' * (10**k) so if you scale you might
>>                    lose information.
>>
>> Well, the equality *might* hold so you *might* lose information,
>> depending on the exact values of L and k.
>>
>>                    Also, note that an n-bit binary fraction may
>>                    require up to n decimal digits to represent it
>>                    accurately.   You need
>>
>>                            2 + floor(   n   )
>>                                      --------       digits.  See Matula[68].
>>                                      log 10
>>                                         2
>>
>>
>>            2. When should we stop printing the integer part?
>>
>> I actually said "fraction part", but in fact the difficulty
>> applies to the integer part as well.
>>
>>            3. How do you round?
>>
>>                    Sidetrack: Why do we round?
>>                            1.  aesthetic - it looks nicer
>>                            2.  rounding preserves information in a way
>>                                that truncation cannot
>>                            3.  portability - if we round correctly then
>>                                numeric input and output (let alone
>>                                programs!) is more easily ported from
>>                                fp format to fp format
>>
>>    The goals of the printing algorithm he presents are:
>>
>>            1.  Preserve information
>>            2.  Use the minimum number of digits (e.g. last digit printed is
>>                    never 0)
>>            3.  Round correctly in ALL cases
>>            4.  Generate digits left to right (other algorithms generate
>>                    digits right to left which is somewhat inconvenient)
>>
>>    The main ideas are:
>>
>>            1. Use integer arithmetic only
>>            2. Use BIGNUM rationals (to avoid scaling problems; also, use
>>                    rational representations)
>>            3. Use explicit interval arithmetic to decide
>>
>>                    * when to stop
>>                    * how to round correctly
>>
>>
>>    The 4 goals of the printing algorithm, expressed in a mathematical
>>    way, are:
>>
>>            1.   -                         +
>>                v  +  v   <   V   <   v + v
>>               ----------            --------
>>                    2                2
>>
>>                +                             -
>>               v   is the successor and v   is the predecessor of a fp
>>               number v.   V (Capital V) is the value printed (refer
>>               to page 119 for a more rigorous description of all this.)
>>
>>               Note that this bound is NOT the same as saying that
>>
>>                    | V - v | < b**(-p)
>>                                -------
>>                                   2
>>
>>                because of discontinuities in representable floating
>>                point numbers (due to jumps in exponents)
>>
>>            2.  Define the output V of the printing algorithm like so:
>>
>>                          H
>>                    V =  Sum   D * (B**i)
>>                        i = N   i
>>
>>                 where H and N are integers and the D (H >= k >= N)
>>                 are digits.                         k
>>
>>                 The goal is: H is as small as possible, and N is
>>                 as large as possible.
>>
>>                                N
>>            3.   | V - v | <=  B         (correct rounding)
>>                              ---
>>                               2
>>
>>
>>            4.  Digits D are generated in order
>>                        K
>>
>>
>>
>>    For the future, Guy Steele said it would be interesting to see if
>>    there is a fast 99% approach.  The printing algorithm needs to be
>>    correct 100% of the time - how about if it is also FAST 99% of the
>>    time?  One idea is to use approximate computations like the Clinger &
>>    Bellerophon algorithms also presented at SIGPLAN 90.
>>
>> You managed to say this part better than I did!
>>
>>    Matula[68] == Matula, David W., "In-and-Out Conversions",
>>    Communications of the ACM 11, 1 (January 1968), 47-50.
>>
>> ...

************************************************************************
*/

#include "headers.h"

#if defined(HAVE_STRTOD)
static int have_strtod = 0;		/* ensure non-empty object file */
					/* because some linkers reject them */
#else

RCSID("$Id: strtod.c,v 1.1 1997/09/01 22:26:35 beebe Exp beebe $")


/*
************************************************************************
For very long digit strings, we may need to temporarily rescale the
number.  This scaling is exact if it is by a power of the floating-point
number base, which is 2, 8, or 16 in virtually all current
floating-point architectures (base 10 is found only in hand-held
calculators).  We choose the smallest common power of 2, 8, and 16:
2**12 == 8**4 == 16**3 = 4096.
************************************************************************
*/

#define COMMON_BASE		4096.0

/*
************************************************************************
The following constant is the limiting value, p, for which (10.0 to p)
is representable on all current floating point architectures.  The
particular value here corresponds to DEC VAX 32-bit F and 64-bit D
floating, and IEEE 32-bit, architectures.  It is used to compute a
coarse, but safe, estimate of the overflow limit.
************************************************************************
*/

#define LIMITING_POSITIVE_POWER_OF_TEN	37

/* We need an interface to pow() which correctly handles pow(x,0.0). */

#define POW(x,y) ((y == 0.0) ? 1.0 : pow((double)(x),(double)(y)))

double
#if STDC
strtod(
const char		*nptr,		/* pointer to number string */
char			**endptr	/* on return, points past end of */
)
#else /* NOT STDC */
strtod(nptr,endptr)
const char		*nptr;		/* pointer to number string */
char			**endptr;	/* on return, points past end of */
#endif /* STDC */
{					/* number, unless == (char**)NULL */
    double		base_scale;	/* used for initial exact scaling */
    int			power_of_base;	/* used for initial exact scaling */
    int			dots_seen;	/* count of decimal points found */
    int			exponent;	/* exponent value */
    int			negative;	/* 0 if positive number */
    double		number;		/* the accumulating number */
    double		overflow_limit;	/* value close to overflow limit */
    register int	power_of_ten;	/* power of ten for fraction */
    double		residual;	/* cumulative error */
    register const char	*q;		/* pointer past end of digits */

    while (isspace(*nptr))
	nptr++;				/* ignore leading whitespace */

    switch (*nptr)			/* set number sign */
    {
    case '-':
	negative = -1;
	nptr++;
	break;

    case '+':
	negative = 0;
	nptr++;
	break;

    default:
	negative = 0;
	break;
    }

    /*
    ********************************************************************
    Collect the entire digit string as a single number, keeping track of
    the scaling required to compensate for the fractional part, and to
    avoid premature overflow with very long digit sequences.
    ********************************************************************
    */

    base_scale = 1.0;
    dots_seen = 0;
    number = 0.0;
    overflow_limit = pow(10.0,(double)LIMITING_POSITIVE_POWER_OF_TEN)/
	COMMON_BASE;
    power_of_base = 0;
    power_of_ten = 0;
    residual = 0.0;

    for (q = nptr; *q && (dots_seen <= 1); q++)
    {					/* accumulate digits */
	if (*q == '.')			/* hit a decimal point */
	    dots_seen++;
	else if (!isdigit(*q))		/* past legal digit string */
	    break;
	else				/* collect digit */
	{
	    double		digit_value;
	    double		rt;
	    double		temp;

	    if (dots_seen > 0)
		power_of_ten++;

	    if (number > overflow_limit) /* rescale to avoid overflow */
	    {
		power_of_base++;
		base_scale *= COMMON_BASE;
		number /= COMMON_BASE;
		residual /= COMMON_BASE;
	    }

	    digit_value = (double)(*q - '0');
	    if (power_of_base > 0)
		digit_value /= base_scale;

	    temp = 10.0 * number;
	    number = temp + digit_value;

	    rt = temp - number;
	    rt += digit_value;
	    residual = 10.0 * residual + rt;
	}
    }

    number += residual + residual;	/* round last significant digit; */
					/* necessary on chopping arithmetics */
					/* such as IBM 360 */

    exponent = 0;

    if ((*q == 'e') || (*q == 'E'))	/* parse exponent */
    {
	register int		negexp;	/* 0 if positive exponent */

	q++;
	if (*q == '+')
	    q++, negexp = 0;
	else if (*q == '-')
	    q++, negexp = 1;
	else				/* no leading sign */
	    negexp = 0;
	for (; isdigit(*q); q++)
	    exponent = 10 * exponent + (int)(*q - '0');
	if (negexp)
	    exponent = -exponent;
    }

    exponent -= power_of_ten;

    /*
    ********************************************************************
    The result here is exact, with correct rounding on all
    floating-point architectures.  Exponent scaling is where errors can
    creep in, and an inaccurate pow() implementation will ruin the final
    result.

    The final scaled result is

    number * base_scale * (10 to `exponent')

    We have to deal with the possibility that (10 to `exponent') may
    underflow or overflow, and so may the product of any two terms, even
    though the complete result is representable.  The extremes of the
    above expression are:

    near underflow: (10 to LIMIT) * (10 to LIMIT) * (10 to -3*LIMIT)
    near overflow:  (10 to LIMIT) * (10 to LIMIT) * (10 to -LIMIT)

    where (10 to LIMIT) is representable.  A safe way to compute this is
    to use frexp() to break the factors into products of the form x * (2
    to y), with x in [1/2,1), and then to use ldexp() to put the product
    together.  Since (10 to -3*LIMIT) may not be representable, we break
    it into smaller parts, with exponent 3*n3 + n4.

    Assume pow() is accurate to p ulps, each multiply is accurate to 0.5
    ulp, and frexp() and ldexp() are exact.  Then the worst-case error
    in the result is (2*p + 2.5) ulps.
    *******************************************************************/

    {
	double		f1, f2, f3, f4, f5;
	int		n1, n2, n3, n4, n5;

	f1 = frexp(number, &n1);
	f2 = frexp(base_scale, &n2);
	n3 = exponent/4;
	n4 = exponent - 3*n3;
	f3 = frexp(POW(10.0,(double)n3), &n3);
	f4 = frexp(POW(10.0,(double)n4), &n4);
	f5 = frexp(f1 * f2 * f3 * f3 * f3 * f4, &n5);
	number = ldexp(f5, n1 + n2 + n3 + n3 + n3 + n4 + n5);
    }

    if (negative)
	number = -number;
    if (endptr != (char**)NULL)
	*endptr = (char*)q;
    return (number);
}
#endif /* defined(HAVE_STRTOD) */
